#!/usr/bin/env python

"""
Sample split function for URLs.
Released into the public domain by Paul Boddie <paul@boddie.net> for
possible further improvement by the wonderful Python community.
Use this software only at your own risk - I disclaim responsibility
for any consequences of the use of this software... including the
correct splitting of URLs in Python. ;-)
"""

from urllib import splittype, splithost, splitnport

def split(url, base=None):

    """
    Split the given 'url' into scheme, host, port and path.
    If the scheme, host and port were found in 'url' then
    include them in the result.
    Otherwise, include the scheme, host and port from the
    'base' parameter. If 'base' is unspecified, then return
    None for the scheme, host and port.
    """

    # NOTE: This isn't verified against W3C recommendations.

    if url.startswith("/"):

        # It's a "rooted" URL.

        if base == None:
            return None, None, None, url

        # Find out something about the base, then.

        scheme, host, port, path = split(base)

        # NOTE: Might want to do something about / characters.

        return scheme, host, port, url

    elif url.startswith("."):

        # The hardest one!

        if base == None:
            return None, None, None, url

        # Check for the first component to see if it really
        # is a relative path.

        url_components = url.split("/")

        if url_components[0] in (".", ".."):

            # Yes, it is a relative URL.
            # Find out something about the base, then.

            scheme, host, port, path = split(base)
            base_components = path.split("/")

            # The base depth is how many components we
            # remove from the end of the base component path.
            # NOTE: The last component will be empty if
            # NOTE: the path ended with a / character.
            # NOTE: If not, we ignore the component anyway;
            # NOTE: this may not be consistent with W3C
            # NOTE: recommendations either.

            base_depth = 1
            url_start = 0

            for url_component in url_components:
                if url_component == "..":
                    base_depth += 1
                    url_start += 1
                elif url_component == ".":
                    url_start += 1
                else:
                    break

            # Get the new base at the calculated depth, making
            # sure that there will always be a leading /
            # character.

            if base_depth < len(base_components):
                new_base_components = base_components[:-base_depth]
            else:
                new_base_components = [""]

            # Get the new path from the first real component,
            # then join it all together.

            new_path = "/".join(
                new_base_components +
                url_components[url_start:])

            return scheme, host, port, new_path

    # It's not relative or rooted, but does it contain anything
    # like a scheme?

    if url.find(":") == -1:
        return None, None, None, url

    # It looks like a genuine URL.

    scheme, rest = splittype(url)
    host_and_port, path = splithost(rest)

    # NOTE: With "file" schemes involving the "//" leading path
    # NOTE: indicator, don't expect splithost to work. For example:
    # NOTE:     file://tmp/myfile

    if host_and_port != None:
        host, port = splitnport(host_and_port)
        if port == -1:
            port = None
    else:
        host, port = None, None

    return scheme, host, port, path

# Ridiculous test cases.

if __name__ == "__main__":

    print 'split("/hello")\n->', \
        split("/hello")
    print 'split("hello")\n->', \
        split("hello")
    print 'split("file:hello")\n->', \
        split("file:hello")
    print 'split("file:/hello")\n->', \
        split("file:/hello")
    print 'split("file://hello")\n->', \
        split("file://hello")
    print 'split("http://www.python.org")\n->', \
        split("http://www.python.org")
    print 'split("http://www.python.org/horse")\n->', \
        split("http://www.python.org/horse")
    print 'split("http://www.python.org/horse/address")\n->', \
        split("http://www.python.org/horse/address")
    print 'split("http://www.python.org/horse/address/")\n->', \
        split("http://www.python.org/horse/address/")
    print 'split("/hello", "http://www.python.org/")\n->', \
        split("/hello", "http://www.python.org/")
    print 'split("/hello", "http://www.python.org/docs")\n->', \
        split("/hello", "http://www.python.org/docs")
    print 'split("/hello", "http://www.python.org/docs/")\n->', \
        split("/hello", "http://www.python.org/docs/")
    print 'split("../hello", "http://www.python.org/docs")\n->', \
        split("../hello", "http://www.python.org/docs")
    print 'split("../hello", "http://www.python.org/docs/")\n->', \
        split("../hello", "http://www.python.org/docs/")
    print 'split("../hello", "http://www.python.org/")\n->', \
        split("../hello", "http://www.python.org/")
    print 'split("../../hello", "http://www.python.org/")\n->', \
        split("../../hello", "http://www.python.org/")
    print 'split("../../hello", "http://www.python.org/a/")\n->', \
        split("../../hello", "http://www.python.org/a/")
    print 'split("../../hello", "http://www.python.org/a/b")\n->', \
        split("../../hello", "http://www.python.org/a/b")
    print 'split("../../hello", "http://www.python.org/a/b/")\n->', \
        split("../../hello", "http://www.python.org/a/b/")
    print 'split("../../hello", "http://www.python.org/a/b/c")\n->', \
        split("../../hello", "http://www.python.org/a/b/c")
    print 'split("../../hello", "http://www.python.org/a/b/c/")\n->', \
        split("../../hello", "http://www.python.org/a/b/c/")
    print 'split("./../../hello", "http://www.python.org/a/b/c/")\n->', \
        split("./../../hello", "http://www.python.org/a/b/c/")
